﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using DistributedGA.Core.Domain;
using DistributedGA.Core.Interface;
using System.Timers;

namespace DistributedGA.Core.Implementation {
  public class WcfPeerListManager : IPeerListManager {

    private string serverString = null;

    private PeerInfo myself = null;

    private Timer timer = null; //sends heartbeat to contact-server

    private Object timerLock = new Object();

    private ChannelFactory<IContactService> myChannelFactory;

    private IContactService client;

    private IContactService heartbeatClient;

    public void Init(PeerInfo source, string contactServerUrl) {
      serverString = contactServerUrl;
      myself = source;
      //Init ChannelFactory and Clients
      var binding = new NetTcpBinding();
      var endpoint = new EndpointAddress(serverString);
      myChannelFactory = new ChannelFactory<IContactService>(binding, endpoint);
      client = myChannelFactory.CreateChannel();
      heartbeatClient = myChannelFactory.CreateChannel();
      //Register Peer
      client.RegisterPeer(source);
      //Start heartbeat timer
      timer = new Timer(1000); //each 5 minutes
      timer.Elapsed += SendHeartbeatToServer;
      timer.Start();
    }

    public List<PeerInfo> GetPeerList() {
      try {
        var allPeers = client.GetPeerList(myself); //maybe timout exception...
        var peersForMessaging = ChoosePeersForMessaging(allPeers);
        //return peersForMessaging;
        return allPeers; //TODO: Enable 10% list communication
      }
      catch { } //if maybe sending failed (because of connection lost, etc.): just ignore
      return new List<PeerInfo>();
    }

    public void SendLogToServer(string msg) {
      client.MakeLog(myself, msg);
    }

    public void Dispose() {
      timer.Stop();
      timer.Dispose();
      ((IClientChannel)client).Close();
      ((IClientChannel)heartbeatClient).Close();
      myChannelFactory.Close();
      client = null;
      myChannelFactory = null;
    }


    private List<PeerInfo> ChoosePeersForMessaging(List<PeerInfo> allPeers) {
      //communicate with 10% of the network
      int noOfPeers = allPeers.Count / 10;
      List<int> indexList = GetRandomItemIndexes(noOfPeers, 0, allPeers.Count - 1);
      List<PeerInfo> res = new List<PeerInfo>();
      foreach (int index in indexList) {
        res.Add(allPeers.ElementAt(index));
      }
      return allPeers;
    }

    private List<int> GetRandomItemIndexes(int noOfItems, int minValue, int maxValue) {
      List<int> res = new List<int>();
      Random rnd = new Random();
      int tmp = -1;
      while (res.Count < noOfItems) {
        tmp = rnd.Next(minValue, maxValue);
        if (!res.Contains(tmp)) {
          res.Add(tmp);
        }
      }
      return res;
    }

    private void SendHeartbeatToServer(object sender, ElapsedEventArgs e) {
      lock (timerLock) {
        try {
          heartbeatClient.UpdateHeartbeat(myself);
        }
        catch { } //nothing to do
      }
    }

  }
}
